/****************************************************************
This is the final project for MAT 259A Winter 2018
Author: April Cai
Usage: Launch the project in Processing 3
Key Interactions: 
Press 1 Center view,
Press 2 90 degree view
Press 3 Side view
Press S Save the frame
Press T Make it spin
Click on each item on the left chart, will display the item content.
*****************************************************************/

import controlP5.*;
import peasy.*;
import processing.opengl.*;

PeasyCam cam;
ControlP5 cp5;
ListBox l;

// Data parameters
Table dataTable;
int rowNums, colNums;
int sum1 = 0;
int sum2 = 0;
int[] seven1 = new int[8];     //array to store the count of 24h relection of happiness
int[] seven2 = new int[8];     //array to store the count of 3 months relection of happiness
String[] sevenCate;  
float[] ratio = new float[7];

// Drawing setting
float overallRadius, panelDistance;
PVector[] Points; 
int presentYear;
float interpolatorYear;
Integrator[] interpolator;
float rotateX, rotateY, rotateZ;
PVector p;

// Color & fonts
color[] colors;
PFont[] fonts;

// Control parameters
int viewMode;
boolean showOtherYear, rotateFree, autoRotate, showGUI, onlyStillFrame, blackOrWhite;
boolean[] titleSelected;
boolean interactWithCate;
boolean[] mouseOnCate, showTheCate;
boolean mouseOnPresentYear;

void setup() {
  //To set it full screen
  fullScreen(P3D);
  //partial screen for test
  //size(1300, 1000, P3D);
  cam = new PeasyCam(this, 250);
  cam.setWheelScale(0.3);
  cam.setDistance(2000);


  //Initialization for data
  //  
  // full 10,000 entries for the happy dataset
  //
  //dataTable = loadTable("cleaned_hm.csv", "header"); 
  //
  // smaller entries for better/easier visual 
  //
  dataTable = loadTable("cleaned_hm_2.csv", "header");
  
  rowNums = dataTable.getRowCount();
  colNums = dataTable.getColumnCount();

  
 
  //calcuate each cate's counts
  for (int i = 0; i < rowNums; i++) {
    TableRow row = dataTable.getRow(i);
    int index = whichCate(row.getString(2));              //to extract each catagory of happiness
    int reflection = whichRelection(row.getString(0));    //reflection time length
    if (reflection == 7) {   //it's the 24hour reflection, count every category
        if (index == 0) seven1[0]++;  //Achievement
        if (index == 1) seven1[1]++;  //Affection
        if (index == 2) seven1[2]++;  //Bonding
        if (index == 3) seven1[3]++;  //Enjoy the moment
        if (index == 4) seven1[4]++;  //Exercise
        if (index == 5) seven1[5]++;  //Leisure
        if (index == 6) seven1[6]++;  //Nature
    }
    if (reflection == 8) {   //it's the 3-month reflection, count every category
        if (index == 0) seven2[0]++;  //same order as above
        if (index == 1) seven2[1]++;
        if (index == 2) seven2[2]++;
        if (index == 3) seven2[3]++;
        if (index == 4) seven2[4]++;
        if (index == 5) seven2[5]++;
        if (index == 6) seven2[6]++;
    }
  }
  //calculate percentage of each 
  //total sum of two relections
  for (int i = 0; i < 7; i++) {
    sum1 += seven1[i];
    sum2 += seven2[i];
  }
  //println(sum1);
  //println(sum2);
  

  sevenCate = new String[7];
  sevenCate[0] = "Achievement";
  sevenCate[1] = "Affection";
  sevenCate[2] = "Bonding";
  sevenCate[3] = "Enjoy the moment";
  sevenCate[4] = "Exercise";
  sevenCate[5] = "Leisure";
  sevenCate[6] = "Nature";


  // Initializaiton for drawing 
  //
  rotateX = 0;
  rotateY = 0;
  rotateZ = 0;

  viewMode = 1;
  //showOtherYear = true;
  rotateFree = true;
  showGUI = true;
  autoRotate = false;
  onlyStillFrame = false;
  interactWithCate = false;


  titleSelected = new boolean[rowNums];
  for (int i = 0; i < rowNums; i++) {
    titleSelected[i] = false;
  }

  mouseOnCate = new boolean[7];
  showTheCate = new boolean[7];  
  for (int i= 0; i<7; i++) {
    mouseOnCate[i] = false;
    showTheCate[i] = true;
  }

  overallRadius = 400;
  panelDistance = 70;
  presentYear = 2017;
  interpolator = new Integrator[4];  
  interpolator[0] = new Integrator(presentYear, 0.4, 0.08);
  interpolator[1] = new Integrator(rotateX, 0.4, 0.16);
  interpolator[2] = new Integrator(rotateY, 0.4, 0.16);
  interpolator[3] = new Integrator(rotateZ, 0.4, 0.16);

  Points = new PVector[8];
  
  for (int i = 0; i < 7; i++) {
    ratio[i] = (float) seven1[i]/sum1;
    //println(ratio[i]);
    float x, y;
    x = overallRadius*cos(i*2*PI/7);
    y = overallRadius*sin(i*2*PI/7);
    Points[i] = new PVector(x, y, 0);
  }


  // Initialize colors & fonts
  colors = new color[10];
  colors[0] = #0A86D0;
  colors[1] = #134ED5;
  colors[2] = #6D5CAF;
  colors[3] = #D2000D;
  colors[4] = #EB6002;
  colors[5] = #FFBC00;
  colors[6] = #019C69;
  colors[7] = #D42020;
  if (!blackOrWhite) {
    colors[9] = #FFFFFF;
    colors[8] = #dbb8ba;
  } else {
    colors[8] = #FFFFFF;
    colors[9] = #000000;
  }

  fonts = new PFont[3];
  fonts[0] = loadFont("BookmanOldStyle-48.vlw");
  fonts[1] = loadFont("BookmanOldStyle-12.vlw");

  controllerSetup();
  noLights();
  smooth();
}

void draw() {
  background(colors[8]);

  for (int i = 0; i < interpolator.length; i++) {   
    interpolator[i].update();
  }
  interpolatorYear = interpolator[0].value;

  // When rotation during the viewmode changing is completed, rotation is restored to be free.
  if (!rotateFree) {
    if (abs(interpolator[1].value - interpolator[1].target)<0.0001 
      && abs(interpolator[2].value - interpolator[2].target)<0.0001 
      && abs(interpolator[3].value - interpolator[3].target)<0.0001) rotateFree = true;
    else cam.setRotations(interpolator[1].value, interpolator[2].value, interpolator[3].value);
  } else {
    setRotationInterpolator(cam.getRotations()[0], cam.getRotations()[1], cam.getRotations()[2]);
  }
  
  for (int i = 0; i < rowNums; i++) {
    if ((l.getItem(i).get("state").toString().equals("true"))) {  
      titleSelected[i] = true;
    } else
      titleSelected[i] = false;
  }

  drawStillFrame();
  if (!onlyStillFrame) {
    for (int i= 0; i < rowNums; i++) {    
      drawTableRow(i);
      
    }
  }

  if (autoRotate) {
    if (viewMode == 1)cam.rotateZ(0.007);
    if (viewMode == 2)cam.rotateX(0.007);
    if (viewMode == 3)cam.rotateY(0.007);
  }

  mouseInteraction();
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  if (showGUI) gui();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
}

int year = 2017;
void drawTableRow(int i) {
  TableRow row = dataTable.getRow(i);
  String theCate = row.getString(2);
  int index = whichCate(theCate);

  // Judgement about itemType
  // If selected, then show.
  //
  boolean shouldShow = false;
  float value = 1000.0;
  if (!shouldShow) {
    float percent = sqrt(sqrt(map(value, seven1[index], seven2[index], 0, 1)));
    
    float percent1 = sqrt(sqrt(map(value, 0, seven1[index], 0, 1)));
    float percent2 = sqrt(sqrt(map(value, 0, seven2[index], 0, 1)));
   
    float percent3 = map(percent1, 0, 1, 0.1, 0.9);
    float percent4 = map(percent2, 0, 1, 0.1, 0.9);
    
    p = new PVector(0, 0, 0);
    //PVector p1 = PVector.lerp(p, Points[index], percent3);
    //PVector p2 = PVector.lerp(p, Points[(index + 1) % 7], percent4);
    PVector p1 = Points[index];
    PVector p2 = Points[(index)%7];
    //PVector p1 = PVector.lerp(Points[index], Points[(index) %7], percent3);
    //PVector p1 = PVector.lerp(PVector.add(Points[index], Points[6]), Points[(index + 1)%7], percent3);
    //PVector p2 = PVector.lerp(Points[index], Points[(index)%7], percent4);
    //PVector p2 = PVector.lerp(PVector.add(Points[index], Points[6]), Points[(index + 1)%7], percent4);    //need more work
    
    float z = (year - interpolatorYear) * panelDistance;
    //println(z);
    
    beginShape(LINES);
    colorMode(HSB, 255);
    color theColor = color(hue(colors[index]), saturation(colors[index])*0.9, brightness(colors[index]));

    // Judgement for each row's diplay mode
    //
    int mode = 3;
    if (showTheCate[index])
      mode = 2;
    else 
    mode = 4;
    if (titleSelected[i]==true) mode = 3;
    else {
      if ( mode != 4) {
        if (presentYear == year) mode = 1;
        else {
          if (showOtherYear ) mode = 2;
          else mode = 4;
        }
      }
    }

    // Display according to its diplay mode
    //
    switch (mode) {

    case 1:
      //
      stroke(theColor, 100);
      strokeWeight(1.5);
      vertex(0, 0, 0);
      vertex(p1.x, p1.y, p1.z);
      vertex(p2.x, p2.y, p2.z);
      stroke(theColor, 250);
      strokeWeight(max(10*percent, 1.5));
      vertex(0, 0, z);
      break;

    case 2:
      stroke(theColor, 40);
      strokeWeight(1);
      //vertex(p.x, p.y, p.z);
      vertex(0, 0, 0);
      vertex(p1.x, p1.y, p1.z);
      vertex(p2.x, p2.y, p2.z);
      stroke(theColor, 110);
      strokeWeight(max(10*percent, 1));
      vertex(0, 0, z);
      break;

    case 3:
      // #3 TitleSelected
      stroke(colors[9], 100);
      strokeWeight(3);
      //vertex(p.x, p.y, p.z);
      //vertex(0, 0, 0);
      vertex(p1.x, p1.y, p1.z);
      vertex(p2.x, p2.y, p2.z);
      stroke(colors[9], 200);
      strokeWeight(max(10*percent, 3));
      //vertex(0, 0, 0);
      break;

    case 4 :
      stroke(hue(colors[index]), saturation(colors[index])*0.1, brightness(colors[index])*0.1, 50);
      strokeWeight(0.5);
      //vertex(p.x, p.y, p.z);
      vertex(0, 0, 0);
      vertex(p1.x, p1.y, p1.z);
      vertex(p2.x, p2.y, p2.z);
      strokeWeight(0.5);
      vertex(0, 0, z);
    }
    endShape();


    // Show the details of a sentence when click on it
    //
    if ((year == presentYear && mode == 4 )|| titleSelected[i]) { 
      pushMatrix();
      translate(p1.x, p1.y, p1.z);
      rotateY(PI/2);
      rotateX(((7-index)+0.5)*2*PI/7 - PI/4);
      rotateZ(PI);
      fill(colors[9], 255);
      textFont(fonts[0], 8);
      textSize(25);
      textAlign(LEFT);
      String title = row.getString(1);   //display the sentence itself
      //rotateY(i * PI/4);
      rotate(i * PI/2 );
      text(title, i , i*3 + 100, i*3 + 200);
      popMatrix();
    }
  }
}

// A mapping between index and cate/ and reflection time
//
int whichCate(String theCate) {
  int index=0;
  if (theCate.equals("achievement"))     index = 0;
  if (theCate.equals("affection"))       index = 1;
  if (theCate.equals("bonding"))         index = 2;
  if (theCate.equals("enjoy_the_moment"))index = 3; 
  if (theCate.equals("exercise"))        index = 4;
  if (theCate.equals("leisure"))         index = 5;
  if (theCate.equals("nature"))          index = 6;
  return index;
}

int whichRelection(String theTime) {
  int index=0;
  if (theTime.equals("24h"))     index = 7;
  if (theTime.equals("3m"))      index = 8;
  return index;
}